@@ -230,6 +230,7 @@ static int s_convert_to_jpeg(MagickWand *magick_wand, request_rec *r, device_tab
 static int s_convert_to_png(MagickWand *maigck_wand, request_rec *r, device_table *spec);
 static int s_convert_to_gif(MagickWand *magick_wand, request_rec *r, device_table *spec);
 static int s_convert_to_bmp(MagickWand *magick_wand, request_rec *r, device_table *spec);
+static int s_chxj_trans_name2(request_rec *r);
 
 
 
@@ -241,22 +242,33 @@ chxj_img_conv_format_handler(request_rec *r)
   char                  *user_agent;
   device_table          *spec;
   chxjconvrule_entry    *entryp;
+  int                   rtn;
 
-  DBG(r, "start chxj_img_conv_format_handler()");
+  DBG(r, "REQ[%X] start chxj_img_conv_format_handler()", (apr_size_t)(unsigned int)r);
   
-  if (r->handler && !STRCASEEQ('c','C',"chxj-picture",r->handler) && !STRCASEEQ('c','C',"chxj-qrcode",r->handler)) {
-    DBG(r, "end chxj_img_conv_format_handler()");
-    return DECLINED;
+  if (! r->handler || 
+      (r->handler && !STRCASEEQ('c','C',"chxj-picture",r->handler) && !STRCASEEQ('c','C',"chxj-qrcode",r->handler))) {
+    DBG(r, "REQ[%X] Response Code:[%d]", (apr_size_t)(unsigned int)r, r->status);
+    s_chxj_trans_name2(r);
+    if (! r->handler) {
+      DBG(r, "REQ[%X] end chxj_img_conv_format_handler() r->handler is null", (apr_size_t)(unsigned int)r);
+      return DECLINED;
+    }
+    if (r->handler && !STRCASEEQ('c','C',"chxj-picture",r->handler) && !STRCASEEQ('c','C',"chxj-qrcode",r->handler)) {
+      DBG(r, "REQ[%X] end chxj_img_conv_format_handler() r->handler is[%s]", (apr_size_t)(unsigned int)r, r->handler);
+      return DECLINED;
+    }
   }
 
   qsp = s_get_query_string_param(r);
   conf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   if (conf == NULL) {
-    DBG(r, "end chxj_img_conv_format_handler() conf is null");
+    DBG(r, "REQ[%X] end chxj_img_conv_format_handler() conf is null", (apr_size_t)(unsigned int)r);
     return DECLINED;
   }
 
   if (STRCASEEQ('c','C',"chxj-qrcode",r->handler) && conf->image == CHXJ_IMG_OFF) {
+    DBG(r, "REQ[%X] end chxj_img_conv_format_handler() chxj-qrcode and ImageEngineOff", (apr_size_t)(unsigned int)r);
     return DECLINED;
   }
 
@@ -286,11 +298,13 @@ chxj_img_conv_format_handler(request_rec *r)
   else
     spec = chxj_specified_device(r, user_agent);
 
-  DBG(r,"found device_name=[%s]", spec->device_name);
-  DBG(r,"User-Agent=[%s]", user_agent);
+  DBG(r,"REQ[%X] found device_name=[%s]", (apr_size_t)(unsigned int)r, spec->device_name);
+  DBG(r,"REQ[%X] User-Agent=[%s]", (apr_size_t)(unsigned int)r, user_agent);
 
 
-  return s_img_conv_format_from_file(r, conf, user_agent, qsp, spec);
+  rtn = s_img_conv_format_from_file(r, conf, user_agent, qsp, spec);
+  DBG(r, "REQ[%X] end chxj_img_conv_format_handler()", (apr_size_t)(unsigned int)r);
+  return rtn;
 }
 
 
@@ -313,11 +327,11 @@ chxj_convert_image(request_rec *r, const char **src, apr_size_t *len)
   char                  *conv_check;
   chxjconvrule_entry    *entryp;
 
-  DBG(r, "start chxj_convert_image()");
+  DBG(r, "REQ[%X] start chxj_convert_image()",(apr_size_t)(unsigned int)r);
 
   conv_check = (char*)apr_table_get(r->headers_in, "CHXJ_IMG_CONV");
   if (conv_check) {
-    DBG(r, "end chxj_exchnage_image() already convert.");
+    DBG(r, "REQ[%X] end chxj_convert_image() already convert.", (apr_size_t)(unsigned int)r);
     return NULL;
   }
 
@@ -325,7 +339,7 @@ chxj_convert_image(request_rec *r, const char **src, apr_size_t *len)
   qsp = s_get_query_string_param(r);
   conf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   if (conf == NULL) {
-    DBG(r, "end chxj_convert_image()");
+    DBG(r, "REQ[%X] end chxj_convert_image()", (apr_size_t)(unsigned int)r);
     return NULL;
   }
 
@@ -353,14 +367,16 @@ chxj_convert_image(request_rec *r, const char **src, apr_size_t *len)
   DBG(r,"found device_name=[%s]", spec->device_name);
   DBG(r, "User-Agent=[%s]", user_agent);
 
-  if (spec->width == 0 || spec->heigh == 0) 
+  if (spec->width == 0 || spec->heigh == 0) {
+    DBG(r, "REQ[%X] end chxj_convert_image() not convert (width or height is 0)", (apr_size_t)(unsigned int)r);
     return NULL;
+  }
 
   dst = s_create_blob_data(r, spec, qsp, (char*)*src, len);
   if (dst == NULL) 
     *len = 0;
 
-  DBG(r, "end chxj_convert_image()");
+  DBG(r, "REQ[%X] end chxj_convert_image()", (apr_size_t)(unsigned int)r);
 
   return dst;
 }
@@ -1937,10 +1953,15 @@ chxj_trans_name(request_rec *r)
   int             ii;
   char            *ext[] = {
           "jpg",
+          "JPG",
           "jpeg",
+          "JPEG",
           "png",
+          "PNG",
           "bmp",
+          "BMP",
           "gif",
+          "GIF",
           "qrc",    /* QRCode出力用ファイルの拡張子 */
           "",
   };
@@ -2065,6 +2086,124 @@ chxj_trans_name(request_rec *r)
 
 
 
+static int
+s_chxj_trans_name2(request_rec *r)
+{
+  apr_finfo_t     st;
+  apr_status_t    rv;
+  mod_chxj_config *conf;
+  int             ii;
+  char            *ext[] = {
+          "jpg",
+          "JPG",
+          "jpeg",
+          "JPEG",
+          "png",
+          "PNG",
+          "bmp",
+          "BMP",
+          "gif",
+          "GIF",
+          "qrc",    /* QRCode出力用ファイルの拡張子 */
+          "",
+  };
+  char     *fname = NULL;
+  char     *filename_sv;
+  int      do_ext_check = TRUE;
+  int      next_ok      = FALSE;
+
+  DBG(r, "REQ[%X] start chxj_trans_name2()", (apr_size_t)(unsigned int)r);
+
+  conf = chxj_get_module_config(r->per_dir_config, &chxj_module);
+
+  if (conf == NULL) {
+    DBG(r, "REQ[%X] end chxj_trans_name2() conf is null[%s]", (apr_size_t)(unsigned int)r, r->uri);
+    return DECLINED;
+  }
+
+  if (conf->image != CHXJ_IMG_ON) {
+    DBG(r, "REQ[%X] end chxj_trans_name2() ImageEngineOff", (apr_size_t)(unsigned int)r);
+    return DECLINED;
+  }
+
+
+  DBG(r,"Match URI[%s]", r->uri);
+
+  if (r->filename == NULL) {
+    DBG(r, "REQ[%X] end chxj_trans_name2() r->filename is null", (apr_size_t)(unsigned int)r);
+    return DECLINED;
+  }
+     
+  filename_sv = r->filename;
+
+  DBG(r,"REQ[%x] r->filename[%s]", (apr_size_t)(unsigned int)r, filename_sv);
+
+  do_ext_check = TRUE;
+  for (ii=0; ii<7-1; ii++) {
+    char* pos = strrchr(filename_sv, '.');
+    if (pos && pos++) {
+      if (strcasecmp(pos, ext[ii]) == 0) {
+        do_ext_check = FALSE;
+        fname = apr_psprintf(r->pool, "%s", filename_sv);
+        break;
+      }
+    }
+  }
+
+  if (do_ext_check) {
+    for (ii=0; ii<7; ii++) {
+      if (strlen(ext[ii]) == 0) {
+        fname = apr_psprintf(r->pool, "%s", filename_sv);
+      }
+      else 
+        fname = apr_psprintf(r->pool, "%s.%s", filename_sv, ext[ii]);
+  
+      DBG(r,"search [%s]", fname);
+  
+      rv = apr_stat(&st, fname, APR_FINFO_MIN, r->pool);
+      if (rv == APR_SUCCESS) {
+        if (st.filetype != APR_DIR)
+          break;
+      }
+  
+      fname = NULL;
+    }
+  }
+  if (fname == NULL) {
+    DBG(r,"NotFound [%s]", r->filename);
+    return DECLINED;
+  }
+  for (ii=0; ii<7-1; ii++) {
+    char* pos = strrchr(fname, '.');
+    if (pos && pos++) {
+      if (strcasecmp(pos, ext[ii]) == 0) {
+        next_ok = TRUE;
+        break;
+      }
+    }
+  }
+
+  if (! next_ok)  {
+    DBG(r,"NotFound [%s]", r->filename);
+    return DECLINED;
+  }
+
+  if (r->handler == NULL || strcasecmp(r->handler, "chxj-qrcode") != 0) {
+    DBG(r,"Found [%s]", fname);
+
+    r->filename = apr_psprintf(r->pool, "%s", fname);
+  
+    if (strcasecmp("qrc", ext[ii]) == 0)
+      r->handler = apr_psprintf(r->pool, "chxj-qrcode");
+    else
+      r->handler = apr_psprintf(r->pool, "chxj-picture");
+  }
+  DBG(r, "REQ[%X] end chxj_trans_name()", (apr_size_t)(unsigned int)r);
+  return OK;
+}
+
+
+
 /**
  * It converts it from QUERYSTRING.
  *
