@@ -379,7 +379,9 @@ chxj_convert(request_rec *r, const char **src, apr_size_t *len, device_table *sp
    * save cookie.
    */
   cookie = NULL;
-  if (entryp->action & CONVRULE_COOKIE_ON_BIT && !(entryp->action & CONVRULE_EMOJI_ONLY_BIT)) {
+  if (entryp->action & CONVRULE_COOKIE_ON_BIT 
+      && !(entryp->action & CONVRULE_EMOJI_ONLY_BIT)
+      && !(entryp->action & CONVRULE_ENVINFO_ONLY_BIT)) {
     switch(spec->html_spec_type) {
     case CHXJ_SPEC_Chtml_1_0:
     case CHXJ_SPEC_Chtml_2_0:
@@ -406,13 +408,14 @@ chxj_convert(request_rec *r, const char **src, apr_size_t *len, device_table *sp
                                                           (apr_size_t *)len);
 
     if (entryp->action & CONVRULE_EMOJI_ONLY_BIT) {
+      if (tmp) {
+        tmp = chxj_node_convert_chxjif_only(r, spec, (const char*)tmp, (apr_size_t *)len);
+      }
+      else {
+        tmp = chxj_node_convert_chxjif_only(r, spec, (const char *)*src, (apr_size_t *)len);
+      }
       if (convert_routine[spec->html_spec_type].emoji_only_converter) {
-        if (tmp) {
-          dst = convert_routine[spec->html_spec_type].emoji_only_converter(r,spec, tmp,*len);
-        }
-        else {
-          dst = convert_routine[spec->html_spec_type].emoji_only_converter(r,spec, *src,*len);
-        }
+        dst = convert_routine[spec->html_spec_type].emoji_only_converter(r,spec, tmp,*len);
         if (dst != NULL) {
           *len = strlen(dst);
         }
@@ -423,29 +426,31 @@ chxj_convert(request_rec *r, const char **src, apr_size_t *len, device_table *sp
         }
       }
       DBG(r, "REQ[%X] end of chxj_convert()(emoji only)", (unsigned int)(apr_size_t)r);
-      return dst;
     }
 
-    if (convert_routine[spec->html_spec_type].converter) {
-      if (tmp)
-        dst = convert_routine[spec->html_spec_type].converter(r, 
-                                                              spec, 
-                                                              tmp, 
-                                                              *len, 
-                                                              len, 
-                                                              entryp, 
-                                                              cookie);
-      else
-        dst = convert_routine[spec->html_spec_type].converter(r,
-                                                              spec, 
-                                                              *src, 
-                                                              *len, 
-                                                              len, 
-                                                              entryp, 
-                                                              cookie);
-    }
-    if (dst && *len) {
-      dst = chxj_conv_z2h(r, dst, len, entryp);
+    if (   !(entryp->action & CONVRULE_EMOJI_ONLY_BIT) 
+        && !(entryp->action & CONVRULE_ENVINFO_ONLY_BIT)) {
+      if (convert_routine[spec->html_spec_type].converter) {
+        if (tmp)
+          dst = convert_routine[spec->html_spec_type].converter(r, 
+                                                                spec, 
+                                                                tmp, 
+                                                                *len, 
+                                                                len, 
+                                                                entryp, 
+                                                                cookie);
+        else
+          dst = convert_routine[spec->html_spec_type].converter(r,
+                                                                spec, 
+                                                                *src, 
+                                                                *len, 
+                                                                len, 
+                                                                entryp, 
+                                                                cookie);
+      }
+      if (dst && *len) {
+        dst = chxj_conv_z2h(r, dst, len, entryp);
+      }
     }
   }
   ap_set_content_length(r, *len);
@@ -1010,6 +1015,7 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
             || STRCASEEQ('j','J',"jpeg2000",        &r->content_type[6])         /* JPEG2000 */
             || STRCASEEQ('j','J',"jpeg2000-image",  &r->content_type[6])         /* JPEG2000 */
             || STRCASEEQ('x','X',"x-jpeg2000-image",&r->content_type[6])         /* JPEG2000 */
+            || STRCASEEQ('x','X',"x-ms-bmp",        &r->content_type[6])         /* BMP */
             || STRCASEEQ('p','P',"png",             &r->content_type[6])         /* PNG */
             || STRCASEEQ('x','X',"x-png",           &r->content_type[6])         /* PNG */
             || STRCASEEQ('g','G',"gif",             &r->content_type[6])))) {     /* GIF */
@@ -1188,7 +1194,9 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
               || STRCASEEQ('x','X',"x-jpeg2000-image",&r->content_type[6])         /* JPEG2000 */
               || STRCASEEQ('p','P',"png",             &r->content_type[6])         /* PNG */
               || STRCASEEQ('x','X',"x-png",           &r->content_type[6])         /* PNG */
+              || STRCASEEQ('x','X',"x-ms-bmp",     &r->content_type[6])         /* BMP */
               || STRCASEEQ('g','G',"gif",             &r->content_type[6]))) {     /* GIF */
+          DBG(r, "REQ[%X] detect convert target:[%s]", (unsigned int)(apr_size_t)r, r->content_type);
           if (ctx->len) {
             char *tmp;
 
@@ -1218,6 +1226,9 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
             if (! ap_is_HTTP_REDIRECT(r->status)) {
               r->status = HTTP_MOVED_TEMPORARILY;
             }
+            ctx->buffer = apr_pstrdup(pool, "");
+            ctx->len    = 0;
+            ap_set_content_length(r, (apr_off_t)ctx->len);
           }
           chxj_cookie_unlock(r,lock);
           s_add_no_cache_headers(r, entryp);
@@ -1226,8 +1237,14 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
                                    (apr_size_t)ctx->len);
         }
         else {
+          ctx->buffer = apr_pstrdup(pool, "");
+          ctx->len    = 0;
+          ap_set_content_length(r, (apr_off_t)ctx->len);
           chxj_cookie_unlock(r, lock);
-
+          s_add_no_cache_headers(r, entryp);
+          rv = pass_data_to_filter(f, 
+                                   (const char *)ctx->buffer, 
+                                   (apr_size_t)ctx->len);
         }
         DBG(f->r, "REQ[%X] end of chxj_output_filter()", (unsigned int)(apr_size_t)r);
         return rv;
@@ -1338,7 +1355,10 @@ chxj_input_handler(request_rec *r)
   apr_pool_create(&pool, r->pool);
 
   dconf      = chxj_get_module_config(r->per_dir_config, &chxj_module);
-  user_agent = (char*)apr_table_get(r->headers_in, "User-Agent");
+  user_agent = (char*)apr_table_get(r->headers_in, CHXJ_HTTP_USER_AGENT);
+  if (!user_agent) {
+    user_agent = (char*)apr_table_get(r->headers_in, HTTP_USER_AGENT);
+  }
   spec       = chxj_specified_device(r, user_agent);
   entryp     = chxj_apply_convrule(r, dconf->convrules);
 
@@ -1545,29 +1565,35 @@ chxj_insert_filter(request_rec *r)
   apr_status_t        rv;
   char                *contentType;
 
-  DBG(r, "start chxj_insert_filter()");
+  DBG(r, "REQ[%X] start chxj_insert_filter()", (apr_size_t)(unsigned int)r);
 
   dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
 
-  user_agent = (char*)apr_table_get(r->headers_in, HTTP_USER_AGENT);
+  /* we get User-Agent from CHXJ_HTTP_USER_AGENT header if any */
+  user_agent = (char *)apr_table_get(r->headers_in, CHXJ_HTTP_USER_AGENT);
+  if (!user_agent) {
+    user_agent = (char*)apr_table_get(r->headers_in, HTTP_USER_AGENT);
+  }
 
   contentType = (char *)apr_table_get(r->headers_in, "Content-Type");
   if (contentType
       && strncasecmp("multipart/form-data", contentType, 19) == 0) {
-    DBG(r, "detect multipart/form-data ==> no target");
+    DBG(r, "REQ[%X] detect multipart/form-data ==> no target", (apr_size_t)(unsigned int)r);
+    DBG(r, "REQ[%X] end chxj_insert_filter()", (apr_size_t)(unsigned int)r);
     return;
   }
 
   spec = chxj_specified_device(r, user_agent);
   entryp = chxj_apply_convrule(r, dconf->convrules);
   if (!entryp) {
-    DBG(r, "end chxj_insert_filter()");
+    DBG(r, "REQ[%X] end chxj_insert_filter()", (apr_size_t)(unsigned int)r);
     return;
   }
   ctx = apr_palloc(r->pool, sizeof(*ctx));
   memset(ctx, 0, sizeof(*ctx));
   if ((rv = apr_pool_create(&ctx->pool, r->pool)) != APR_SUCCESS) {
-    ERR(r, "failed: new pool create. rv:[%d]", rv);
+    ERR(r, "%s:%d: failed: new pool create. rv:[%d]", __FILE__,__LINE__,rv);
+    DBG(r, "REQ:[%X] end chxj_insert_filter()", (apr_size_t)(unsigned int)r);
     return;
   }
   ctx->entryp = entryp;
@@ -1576,7 +1602,8 @@ chxj_insert_filter(request_rec *r)
   ctx->buffer[0] = 0;
 
   if (!entryp || !(entryp->action & CONVRULE_ENGINE_ON_BIT)) {
-    DBG(r,"EngineOff");
+    DBG(r,"REQ[%X] EngineOff", (apr_size_t)(unsigned int)r);
+    DBG(r, "REQ[%X] end chxj_insert_filter()", (apr_size_t)(unsigned int)r);
     return;
   }
 
@@ -1595,16 +1622,17 @@ chxj_insert_filter(request_rec *r)
     break;
 
   default:
+    DBG(r, "REQ[%X] end chxj_insert_filter() Unknown spec type.", (apr_size_t)(unsigned int)r);
     return;
   }
 
 
   if (! apr_table_get(r->headers_in, "X-Chxj-Forward")) {
     ap_add_output_filter("chxj_output_filter", ctx, r, r->connection);
-    DBG(r, "added Output Filter");
+    DBG(r, "REQ[%X] added Output Filter", (apr_size_t)(unsigned int)r);
   }
 
-  DBG(r, "end   chxj_insert_filter()");
+  DBG(r, "REQ[%X] end chxj_insert_filter()", (apr_size_t)(unsigned int)r);
 }
 
 
@@ -1675,6 +1703,7 @@ chxj_create_per_dir_config(apr_pool_t *p, char *arg)
 #endif
   conf->forward_url_base = NULL;
   conf->forward_server_ip = NULL;
+  conf->allowed_cookie_domain = NULL;
 
   if (arg == NULL) {
     conf->dir                  = NULL;
@@ -1719,6 +1748,7 @@ chxj_merge_per_dir_config(apr_pool_t *p, void *basev, void *addv)
   mrg->new_line_type    = NLTYPE_NIL;
   mrg->forward_url_base = NULL;
   mrg->forward_server_ip = NULL;
+  mrg->allowed_cookie_domain = NULL;
 
   mrg->dir = apr_pstrdup(p, add->dir);
 
@@ -1937,6 +1967,13 @@ chxj_merge_per_dir_config(apr_pool_t *p, void *basev, void *addv)
   else if (base->forward_url_base) {
     mrg->forward_url_base = base->forward_url_base;
   }
+
+  if (add->allowed_cookie_domain) {
+    mrg->allowed_cookie_domain = add->allowed_cookie_domain;
+  }
+  else {
+    mrg->allowed_cookie_domain = base->allowed_cookie_domain;
+  }
   return mrg;
 }
 
@@ -2310,6 +2347,10 @@ cmd_convert_rule(cmd_parms *cmd, void *mconfig, const char *arg)
       if (strcasecmp(CONVRULE_EMOJI_ONLY_CMD, action) == 0) {
         newrule->action |= CONVRULE_EMOJI_ONLY_BIT;
       }
+      else
+      if (strcasecmp(CONVRULE_ENVINFO_ONLY_CMD, action) == 0) {
+        newrule->action |= CONVRULE_ENVINFO_ONLY_BIT;
+      }
       break;
 
     case 'C':
@@ -2739,6 +2780,24 @@ cmd_set_forward_server_ip(
   return NULL;
 }
 
+static const char *
+cmd_allowed_cookie_domain(
+  cmd_parms   *cmd,
+  void        *mconfig,
+  const char  *arg)
+{
+  mod_chxj_config *dconf;
+
+  if (strlen(arg) > 255)
+    return "mod_chxj: ChxjAllowedCookieDomain is too long.";
+
+  dconf = (mod_chxj_config *)mconfig;
+
+  dconf->allowed_cookie_domain = apr_pstrdup(cmd->pool, arg);
+
+  return NULL;
+}
+
 static const char *
 cmd_set_new_line_type(
   cmd_parms   *UNUSED(cmd), 
@@ -2919,6 +2978,12 @@ static const command_rec cmds[] = {
     NULL,
     OR_ALL,
     "The forward server ip(default: this server ip)"),
+  AP_INIT_TAKE1(
+    "ChxjAllowedCookieDomain",
+    cmd_allowed_cookie_domain,
+    NULL,
+    OR_ALL,
+    "Domain that permits parameter addition for cookie besides hostname.(Default:hostname Only)"),
   {NULL}
 };
 
