@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2008 Atsushi Konno All rights reserved.
+ * Copyright (C) 2005-2009 Atsushi Konno All rights reserved.
  * Copyright (C) 2005 QSDN,Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -25,7 +25,7 @@
 
 
 void
-qs_log(Doc* doc, int log_level,const char* f, int l, char* msg) 
+qs_log(Doc *doc, int log_level,const char *f, int l, char *msg) 
 {
   switch(log_level) {
   case QX_LOG_DEBUG:
@@ -79,16 +79,17 @@ qs_log(Doc* doc, int log_level,const char* f, int l, char* msg)
     }
     while(0);
 #endif
-    if (doc->r) {
+    if (doc && doc->r) {
       chxj_log_rerror(f,l, APLOG_CRIT, 0, doc->r, msg);
     }
-    qs_all_free(doc, QX_LOGMARK);
-    _exit(0);
+    else {
+      fprintf(stderr, "%s\n", msg);
+    }
   }
 }
 
 void
-qs_log_int(Doc* UNUSED(doc), int UNUSED(log_level),const char* UNUSED(f), int UNUSED(l), char* UNUSED(msg), int UNUSED(val)) 
+qs_log_int(Doc *UNUSED(doc), int UNUSED(log_level),const char *UNUSED(f), int UNUSED(l), char *UNUSED(msg), int UNUSED(val)) 
 {
 #if 0
 #ifdef USE_LOG
